Installation
===

## Dependencies

To build examples, have HyPro built somewhere on your system. Instructions on how to get and compile can be found on [github](https://github.com/hypro/hypro).


## Build examples

To build an example, first create a directory to build, e.g., via `mkdir build`. From the build folder invoke

```console
/build$ cmake ..
```

to initialize the CMake environment, which allows to build example files contained in src/
Each example file has its own cmake-target, i.e. each file should contain a
main() function and each example can be build by invoking

```console
/build$ make example
```

where the according cpp-file is "example.cpp". The built executables will be
put in build/bin/.

Examples
===

The provided examples can be modified to see the effects of different operations and parameter settings.
In the following, we present for each example some possible adjustments for you to try out.

## Box (`box.cpp`)

The example shows how to perform operations on boxes.

### Possible Modifications

* You can try different boxes for the operations intersection
* You can use other operations, e.g., Minkowski sum

## Support Function (`supportFunction.cpp`)

The example shows the symbolic character of support functions.
The influence of the template for templated-evaluation is highlighed, you can try different levels of precision by testing different numbers of template directions (uniformly distributed).

### Call (to use 5 uniformly distributed directions)
```console
build$ ./bin/supportFunction 5
```


### Possible Modifications

* You can modify the number of directions used for evaluation, in fact, the first argument passed corresponds to the number of template directions (uniformly distributed in 2D).

## Simplified Reachability Analysis (`reachability.cpp`)

This example manually replays all the steps to compute time successor steps.
The used errors for the over-approximation are hard-coded and manually chosen, the example indicates what operations are needed throughout the computation.

## Reachability (`reacher.cpp`)

The example shows how to set up a very basic reachability analysis method with HyPro.
The example reads an input file in Flow*-format with the provided parser and calls a basic reachability analysis method.
Some exemplary input files are provided in the folder `models/`.

### Call
```console
build/$ ./bin/reacher <MODEL>
```
where `<MODEL>` is the relative path to the model file used.

### Possible Modifications

Settings which should output results in reasonable time (about 20-30 sec. at most)

* Support Functions, delta >= 0.1, Full aggregation or clustering 5
* Boxes