#include <hypro/representations/GeometricObjectBase.h>
#include <hypro/util/plotting/Plotter.h>

using namespace hypro;

int main(int argc, char **argv) {
  using Number = double;

  // plotting
  Plotter<Number> &plotter = Plotter<Number>::getInstance();
  plotting::gnuplotSettings settings = plotter.settings();
  settings.keepAspectRatio = true;
  settings.linewidth = 1.5;
  settings.fill = true;
  plotter.updateSettings(settings);
  plotter.setFilename("box");
  std::vector<Point<Number>> points;

  // b1: [1,3] x [1,3]
  Box<Number> b1 =
      Box<Number>(std::make_pair(Point<Number>({1, 1}), Point<Number>({3, 3})));

  std::vector<carl::Interval<Number>> intervals;
  // intervals.push_back(carl::Interval<Number>(2, 4));
  // intervals.push_back(carl::Interval<Number>(2, 4));
  intervals.push_back(carl::Interval<Number>(4, 5));
  intervals.push_back(carl::Interval<Number>(4, 5));

  // b2: [4,5] x [4,5]
  Box<Number> b2 = Box<Number>(intervals);

  Box<Number> intersected = b1.intersect(b2);
  Box<Number> added = b1.minkowskiSum(b2);

  unsigned b1Id =
      plotter.addObject(b1.vertices(), plotting::colors[plotting::green]);
  unsigned b2Id =
      plotter.addObject(b2.vertices(), plotting::colors[plotting::blue]);
  // unsigned b3Id = plotter.addObject(intersected.vertices());
  unsigned b3Id =
      plotter.addObject(added.vertices(), plotting::colors[plotting::orange]);

  // create actual gnuplot file, the second parameter tells the plotter to try
  // to invoke gnuplot directly
  plotter.plot2d(PLOTTYPE::png, true);

  /*
  plotter.clear();
  plotter.setFilename("box_trafo");
  Box<Number> b4{std::make_pair(Point<Number>{1, 1}, Point<Number>{2, 2})};
  matrix_t<Number> rotation = matrix_t<Number>(2, 2);
  rotation << 0.707, -0.707, 0.707, 0.707;

  VPolytope<Number> vPoly = Converter<Number>::toVPolytope(b4);
  auto vPoly_rotated = vPoly.linearTransformation(rotation);
  Box<Number> b4_rotated = b4.linearTransformation(rotation);

  unsigned b4Id = plotter.addObject(b4.vertices());
  unsigned b4_rotatedId = plotter.addObject(b4_rotated.vertices());
  unsigned vPoly_rotatedId = plotter.addObject(vPoly_rotated.vertices());

  plotter.setObjectColor(b4Id, plotting::colors[plotting::green]);
  plotter.setObjectColor(b4_rotatedId, plotting::colors[plotting::blue]);
  plotter.setObjectColor(vPoly_rotatedId, plotting::colors[plotting::orange]);

  // create actual gnuplot file
  plotter.plot2d(PLOTTYPE::png);
  */

  return 0;
}
