#include <bits/c++config.h>
#include <hypro/algorithms/reachability/Reach.h>
#include <hypro/datastructures/reachability/ReachTreev2.h>
#include <hypro/datastructures/reachability/ReachTreev2Util.h>
#include <hypro/datastructures/reachability/TreeTraversal.h>
#include <hypro/parser/antlr4-flowstar/ParserWrapper.h>
#include <hypro/types.h>
#include <hypro/util/plotting/Plotter.h>
#include <ostream>
#include <string>
#include <vector>

int main(int argc, char *argv[]) {
  // read command line arguments
  std::vector<std::string> arguments;

  if (argc > 1) {
    arguments.assign(argv + 1, argv + argc);
  }

  // aliasses
  using Number = double;
  // using Representation = hypro::Box<Number>;
  using Representation =
      hypro::SupportFunctionT<Number, hypro::Converter<Number>,
                              hypro::NoBoxReduction>;

  // parse input file
  auto [automaton, settings] = hypro::parseFlowstarFile<Number>(arguments[0]);

  // set up settings
  hypro::FixedAnalysisParameters fixedSettings{
      static_cast<size_t>(settings.jumpDepth), settings.timeBound};
  hypro::AnalysisParameters analysisSettings{settings.timeStep,
                                             hypro::AGG_SETTING::AGG, 1};

  // set up reachability tree
  auto roots = hypro::makeRoots<Representation>(automaton);

  // create RA-instance
  hypro::reachability::Reach<Representation> reacher{automaton, fixedSettings,
                                                     analysisSettings, roots};

  std::cout << "Reachability analysis ... " << std::flush;

  auto analysisResult = reacher.computeForwardReachability();
  std::size_t numberSegments = 0;
  for (const auto &node : preorder(roots)) {
    std::for_each(node.getFlowpipe().begin(), node.getFlowpipe().end(),
                  [&numberSegments](auto &segment) { ++numberSegments; });
  }

  std::cout << "done (" << numberSegments << " segments)." << std::endl;

  // prepare for plotting
  auto &plotter = hypro::Plotter<Number>::getInstance();
  hypro::plotting::gnuplotSettings pltSettings = plotter.settings();
  pltSettings.keepAspectRatio = false;
  pltSettings.linewidth = 1.5;
  pltSettings.fill = true;
  plotter.updateSettings(pltSettings);
  plotter.setFilename("reachability");

  std::cout << "Plotting sets ... " << std::flush;
  // collect state sets by traversing the reachability tree
  // attention: Plot first two dimensions here
  for (const auto &node : preorder(roots)) {
    std::for_each(node.getFlowpipe().begin(), node.getFlowpipe().end(),
                  [&plotter](auto &segment) {
                    plotter.addObject(segment.projectOn({0, 1}).vertices());
                  });
  }

  plotter.plot2d(hypro::PLOTTYPE::png, true);
  std::cout << "done." << std::endl;

  return 0;
}