#pragma once
#include <hypro/datastructures/HybridAutomaton/HybridAutomaton.h>

template <typename Number> hypro::HybridAutomaton<Number> getThermostat() {
  using namespace hypro;
  using Matrix = matrix_t<Number>;
  using Vector = vector_t<Number>;

  HybridAutomaton<Number> automaton{};

  // create locations
  Location<Number> *loc_on = automaton.createLocation();
  loc_on->setName("loc_on");

  // fix variable order: t, x, const

  // set up flow for location "on": x' = x' = 0.1*(35 - x), t' = 1
  Matrix flow_matrix = Matrix::Zero(3, 3);

  flow_matrix << 0, 0, 1, 0, -0.1, 3.5, 0, 0, 0;
  loc_on->setFlow(linearFlow(flow_matrix));

  // dummy transition
  Transition<Number> *self_loop = loc_on->createTransition(loc_on);
  // dummy guard: x >= 24
  Matrix guard_matrix = Matrix::Zero(1, 2);
  Vector guard_vector = Vector::Zero(1);
  guard_matrix << 0, -1;
  guard_vector << -24;
  self_loop->setGuard(Condition(guard_matrix, guard_vector));
  // dummy reset: t := 0
  Matrix reset_matrix = Matrix::Identity(2, 2);
  Vector reset_vector = Vector::Zero(2);
  reset_matrix(0, 0) = 0;
  self_loop->setReset(Reset(reset_matrix, reset_vector));

  return automaton;
}